#include "config.h"
#include "vm.h"
#include "puts.h"
#include "nds32_intrinsic.h"

extern unsigned int _ILM_BASE;
extern unsigned int _ILM_SIZE;
extern unsigned int __text_start;
extern unsigned int __text_end;

extern unsigned int _DLM_BASE;
extern unsigned int __data_lmastart;
extern unsigned int __data_start;
extern unsigned int edata;


unsigned int g_ilm_size, g_dlm_size;


char * __itoa(int value, char *string, int radix)
{
  char tmp[33];
  char *tp = tmp;
  int i;
  unsigned v;
  int sign;
  char *sp;

  if (radix > 36 || radix <= 1)
  {
    //__set_errno(EDOM);
    puts("itoa error ....\n");
    return 0;
  }

  sign = (radix == 10 && value < 0);
  if (sign)
    v = -value;
  else
    v = (unsigned)value;
  while (v || tp == tmp)
  {
    i = v % radix;
    v = v / radix;
    if (i < 10)
      *tp++ = i+'0';
    else
      *tp++ = i + 'a' - 10;
  }

  /*if (string == 0)
    string = (char *)malloc((tp-tmp)+sign+1);*/
    
  sp = string;

  if (sign)
    *sp++ = '-';
  while (tp > tmp)
    *sp++ = *--tp;
  *sp = 0;
  return string;
}

void initValue(char * pValue, int count) 
{
    int i;
    for (i=0; i < count; i++)
         pValue[i]='\0';	
}	

void printS(const char *msg, unsigned int value, int radix)
{
	char str_value[32];
	char *pValue;
	
	initValue(str_value, 32);
	pValue = __itoa(value, str_value, radix);
	puts(msg);
	puts((const char*)pValue);
	puts("\n");	
	
}	


void copyTextToILM()
{
	volatile unsigned int source_addr, ilm_base, size;

	puts("Copying .text section to ILM.......................\n");
	
	/*
	 * source_addr = source address
     * ilm_base = destination address
     * size = size to copy
     */ 
	//source_addr = &__text_start;
	//__asm__ volatile ("1:\n\t");
    //__asm__ volatile ("b 1b\n\t");
    
	source_addr = (volatile unsigned int) 0 ;
	ilm_base = &_ILM_BASE;
	//size = &_ILM_SIZE;
	size = g_ilm_size;
	//size = (volatile unsigned int)&__text_end - (volatile unsigned int)&__text_start;
	
	if (size > 64*1024) {
		puts("Text section size exceeds ILM size (64KB)........\n");
		return;
    }	
	
	/* Set ILM base and enable it*/
	ilm_base = ilm_base + 1;
	/*mtsr	$r1, $ilmb*/
	__nds32__mtsr(ilm_base, NDS32_SR_ILMB);
	__nds32__dsb();
	ilm_base = ilm_base - 1;
	
	volatile unsigned int *src = (volatile unsigned int *) source_addr;
	volatile unsigned int *dest = (volatile unsigned int *) ilm_base;
	
	while (size > 0) {
		*dest++ = *src++;
		 size = size - 4;
    }
    //memcpy( (unsigned int *)source_addr, (unsigned int *)ilm_base, size);
	
    /* Set ILM base to SDRAM and enable it*/
	//size = (volatile unsigned int)&__text_start + 1;
	size = size + 1;
	__nds32__mtsr(size, NDS32_SR_ILMB);
	__nds32__dsb();

}

void copyDataToDLM() {
	
	volatile unsigned int source_addr, dlm_base, size;
	
	// Copy data section to DLM 
	source_addr = &__data_lmastart;
	dlm_base    = &_DLM_BASE;
	size = (volatile unsigned int) &edata - (volatile unsigned int)&__data_start;
	
	if (size <= 0)
	    return;
	
	/* Set DLM base and enable it*/
	dlm_base = dlm_base + 1;
	__nds32__mtsr(dlm_base, NDS32_SR_DLMB);
	__nds32__dsb();
	
	dlm_base = dlm_base - 1;
	
	volatile unsigned int *src = (volatile unsigned int *)(source_addr);
	volatile unsigned int *dest = (volatile unsigned int *)(dlm_base);
	
	while (size > 0) {
		*dest++ = *src++;
		 size = size - 4;
    }
    
    /* Set DLM base to __data_start and enable it*/
	size = (volatile unsigned int)&__data_start + 1;
	__nds32__mtsr(size, NDS32_SR_DLMB);
	__nds32__dsb();
   		
}	
	
 void executeOnILM()
{
	int i;
	
	puts("Execute for loop i on ILM.......................\n");
	
	for (i=0; i < 65536; i++)
	     i=i+1; 
}	 

int main()
{
	if (LOWLEVEL_INIT==1)
		uart_init();
		
	unsigned int icm_cfg, dcm_cfg;
	unsigned int ilmb, dlmb, ilm_size, dlm_size;	
		
	icm_cfg = __nds32__mfsr(NDS32_SR_ICM_CFG);
	printS("ICM_CFG = 0x", icm_cfg, 16);
	
	if(icm_cfg & 0x1C00)
	    puts("The system has ILM...\n");    
	else {
		puts("The system has no ILM...\n");
		return 0;
    }	
    
    dcm_cfg = __nds32__mfsr(NDS32_SR_DCM_CFG);
	printS("DCM_CFG = 0x", dcm_cfg, 16);
	if (dcm_cfg & 0x1C00) 
	    puts("The system has DLM...\n");
	else {
		puts("The system has no DLM...\n");
		return 0;
    }	
	        	
    ilmb = __nds32__mfsr(NDS32_SR_ILMB);
    dlmb = __nds32__mfsr(NDS32_SR_DLMB);
    
    ilm_size = ((ilmb & 0x1E) >> 1);
    printS("ilm_size = 0x", ilm_size, 16);
    
    if (ilm_size == 0)
        g_ilm_size = 1024 * 4;
    else if (ilm_size == 1)
    	g_ilm_size = 1024 * 8;   
    else if (ilm_size == 2)	
    	g_ilm_size =1024 * 16;
    else if (ilm_size == 3)	
    	g_ilm_size =1024 * 32;
    else if (ilm_size == 4)	
    	g_ilm_size =1024 * 64;
    else if (ilm_size == 5)	
    	g_ilm_size =1024 * 128;
    else if (ilm_size == 6)	
    	g_ilm_size =1024 * 256;	
    else if (ilm_size == 7)	
    	g_ilm_size =1024 * 512;	
    else if (ilm_size == 8)	
    	g_ilm_size =1024 * 1024;
    else if (ilm_size == 9)	
    	g_ilm_size =1024;
    else if (ilm_size == 10)	
    	g_ilm_size =1024 * 2;
    else if (ilm_size == 15)
    	g_ilm_size =0;		 					
    else if ( (ilm_size == 11) || (ilm_size == 12) || (ilm_size == 13) || (ilm_size == 14))	
        g_ilm_size = -1; //Reserved
    else
    	g_ilm_size = -1;      
    
	printS("g_ilm_size = 0x", g_ilm_size, 16);
	
	
	dlm_size = ((dlmb & 0x1E) >> 1);
	printS("dlm_size = 0x", dlm_size, 16);
	
    if (dlm_size == 0)
        g_dlm_size = 1024 * 4;
    else if (dlm_size == 1)
    	g_dlm_size = 1024 * 8;   
    else if (dlm_size == 2)	
    	g_dlm_size =1024 * 16;
    else if (dlm_size == 3)	
    	g_dlm_size =1024 * 32;
    else if (dlm_size == 4)	
    	g_dlm_size =1024 * 64;
    else if (dlm_size == 5)	
    	g_dlm_size =1024 * 128;
    else if (dlm_size == 6)	
    	g_dlm_size =1024 * 256;	
    else if (dlm_size == 7)	
    	g_dlm_size =1024 * 512;	
    else if (dlm_size == 8)	
    	g_dlm_size =1024 * 1024;
    else if (dlm_size == 9)	
    	g_dlm_size =1024;
    else if (dlm_size == 10)	
    	g_dlm_size =1024 * 2;
    else if (dlm_size == 15)
    	g_dlm_size =0;		 					
    else if ( (dlm_size == 11) || (dlm_size == 12) || (dlm_size == 13) || (dlm_size == 14))	
        g_dlm_size = -1; //Reserved
    else
    	g_dlm_size = -1;       
    
	printS("g_dlm_size = 0x", g_dlm_size, 16);
	
	puts("Instruction/Data Local Memory sample...............\n");
	copyTextToILM();
	executeOnILM();
	copyDataToDLM();
	puts("Completed...............\n");
	while(1);
}
