//vm.h

#ifndef VM_H
#define VM_H

#include "ag101.h"

//type shortcuts
typedef unsigned char  u8;
typedef unsigned short u16;
typedef unsigned int   u32;
typedef volatile unsigned int vu32;

//
// Interrupt Controller, INTC010
//

#define IC_ISRC     (*((vu32*)(0x000 + INTC_BASE)))
#define IC_IENA     (*((vu32*)(0x004 + INTC_BASE)))
#define IC_ICLR     (*((vu32*)(0x008 + INTC_BASE)))
#define IC_IMOD     (*((vu32*)(0x00c + INTC_BASE)))
#define IC_ILEV     (*((vu32*)(0x010 + INTC_BASE)))
#define IC_ISTA     (*((vu32*)(0x014 + INTC_BASE)))

#define IC_FSRC     (*((vu32*)(0x020 + INTC_BASE)))
#define IC_FENA     (*((vu32*)(0x024 + INTC_BASE)))
#define IC_FCLR     (*((vu32*)(0x028 + INTC_BASE)))
#define IC_FMOD     (*((vu32*)(0x02c + INTC_BASE)))
#define IC_FLEV     (*((vu32*)(0x030 + INTC_BASE)))
#define IC_FSTA     (*((vu32*)(0x034 + INTC_BASE)))

#define IC_REV      (*((vu32*)(0x050 + INTC_BASE)))
#define IC_INPNO    (*((vu32*)(0x054 + INTC_BASE)))
#define IC_IDBL     (*((vu32*)(0x058 + INTC_BASE)))
#define IC_FDBL     (*((vu32*)(0x05c + INTC_BASE)))

// interrupt sources
#define IRQ_LCDC    (4) //extension of A320D

#define IC_GPIO     (1 << IRQ_GPIO)

#define IC_TIMER1   (1 << IRQ_TIMER1)
#define IC_TIMER2   (1 << IRQ_TIMER2)
#define IC_TIMER3   (1 << IRQ_TIMER3)

#define IC_DMA     (1 << IRQ_CPE_AHB_DMA)

#define IC_UART1    (1 << IRQ_UART1)
#define IC_UART2    (1 << IRQ_UART2)

#define IC_LCDC     (1 << IRQ_LCDC)
#define IC_ETHC     (1 << IRQ_MAC)

//
// ARM's abstract reference interrupt controller
//

// IRQ
#define IC0_ISTA    (*((vu32*)(0x100 + INTC_BASE)))
#define IC0_IRAW    (*((vu32*)(0x104 + INTC_BASE)))
#define IC0_IENA    (*((vu32*)(0x108 + INTC_BASE)))
#define IC0_ICLR    (*((vu32*)(0x10c + INTC_BASE)))
#define IC0_ISOFT   (*((vu32*)(0x110 + INTC_BASE)))

// FIQ
#define IC0_FSTA    (*((vu32*)(0x200 + INTC_BASE)))
#define IC0_FRAW    (*((vu32*)(0x204 + INTC_BASE)))
#define IC0_FENA    (*((vu32*)(0x208 + INTC_BASE)))
#define IC0_FCLR    (*((vu32*)(0x20c + INTC_BASE)))

// interrupt sources
#define IC_TIMER0   (1 << 0)

//
// Timer 0 ~ 3
//

// Timer 0, SID's implementation
//  16-bit module, down counter type
//  Control Register
//      xxxx xxxx emxx ppxx
//                ||   pre-scale:  0/over 1, 1/over 16, 2/over 256, 3/undefined
//                |mode:           1/periodic, 0/free_running
//                enable:          1/enable, 0/disable
#define TM0_LOAD   (*((vu32*)(0x100 + TIMER_BASE)))
#define TM0_VAL    (*((vu32*)(0x104 + TIMER_BASE)))
#define TM0_CTRL   (*((vu32*)(0x108 + TIMER_BASE)))
#define TM0_CLR    (*((vu32*)(0x10c + TIMER_BASE)))

#define CTRL_ENA   (0x1 << 7) //1/enable
#define CTRL_MODE  (0x1 << 6) //1/periodic
#define CTRL_PRES  (0x3 << 2) //0 ~ 2

// Timer 1 ~ 3, ATC010
#define TM1_CNTR  (*((vu32*)(0x000 + TIMER_BASE)))
#define TM1_LOAD  (*((vu32*)(0x004 + TIMER_BASE)))
#define TM1_MTCH1 (*((vu32*)(0x008 + TIMER_BASE)))
#define TM1_MTCH2 (*((vu32*)(0x00c + TIMER_BASE)))

#define TM2_CNTR  (*((vu32*)(0x010 + TIMER_BASE)))
#define TM2_LOAD  (*((vu32*)(0x014 + TIMER_BASE)))
#define TM2_MTCH1 (*((vu32*)(0x018 + TIMER_BASE)))
#define TM2_MTCH2 (*((vu32*)(0x01c + TIMER_BASE)))

#define TM3_CNTR  (*((vu32*)(0x020 + TIMER_BASE)))
#define TM3_LOAD  (*((vu32*)(0x024 + TIMER_BASE)))
#define TM3_MTCH1 (*((vu32*)(0x028 + TIMER_BASE)))
#define TM3_MTCH2 (*((vu32*)(0x02c + TIMER_BASE)))

#define TM_CR     (*((vu32*)(0x030 + TIMER_BASE)))
#define TM_STA    (*((vu32*)(0x034 + TIMER_BASE)))
#define TM_MSK    (*((vu32*)(0x038 + TIMER_BASE)))

// TmCR
#define TM_CR_TM1ENA (0x1 << 0)  //1/enable
#define TM_CR_TM1CLK (0x1 << 1)  //1/EXT1CLK, 0/PCLK
#define TM_CR_TM1OEN (0x1 << 2)  //1/enable overflow interrupt
#define TM_CR_TM2ENA (0x1 << 3)  //1/enable
#define TM_CR_TM2CLK (0x1 << 4)  //1/EXT1CLK, 0/PCLK
#define TM_CR_TM2OEN (0x1 << 5)  //1/enable overflow interrupt
#define TM_CR_TM3ENA (0x1 << 6)  //1/enable
#define TM_CR_TM3CLK (0x1 << 7)  //1/EXT1CLK, 0/PCLK
#define TM_CR_TM3OEN (0x1 << 8)  //1/enable overflow interrupt
#define TM_CR_TM1UP  (0x1 << 9)  //1/up count
#define TM_CR_TM2UP  (0x1 << 10) //1/up count
#define TM_CR_TM3UP  (0x1 << 11) //1/up count

// IntrState
#define TM_STA_TM1M1 (0x1 << 0)
#define TM_STA_TM1M2 (0x1 << 1)
#define TM_STA_TM1OV (0x1 << 2)
#define TM_STA_TM2M1 (0x1 << 3)
#define TM_STA_TM2M2 (0x1 << 4)
#define TM_STA_TM2OV (0x1 << 5)
#define TM_STA_TM3M1 (0x1 << 6)
#define TM_STA_TM3M2 (0x1 << 7)
#define TM_STA_TM3OV (0x1 << 8)

// IntrMask
#define TM_MSK_TM1M1 TM_STA_TM1M1
#define TM_MSK_TM1M2 TM_STA_TM1M2
#define TM_MSK_TM1OV TM_STA_TM1OV
#define TM_MSK_TM2M1 TM_STA_TM2M1
#define TM_MSK_TM2M2 TM_STA_TM2M2
#define TM_MSK_TM2OV TM_STA_TM2OV
#define TM_MSK_TM3M1 TM_STA_TM3M1
#define TM_MSK_TM3M2 TM_STA_TM3M2
#define TM_MSK_TM3OV TM_STA_TM3OV

//
// UART 0 ~ 2
//

#define UR1_BASE (CPE_UART1_BASE)

// UART 0, SID's implementation
//   refer to PC16550D datasheet
#define UR0_RBR (*((vu32*)(0x100 + UR1_BASE)))
#define UR0_THR (*((vu32*)(0x100 + UR1_BASE)))
#define UR0_IER (*((vu32*)(0x104 + UR1_BASE)))
#define UR0_IIR (*((vu32*)(0x108 + UR1_BASE)))
#define UR0_FCR (*((vu32*)(0x108 + UR1_BASE)))
#define UR0_LCR (*((vu32*)(0x10c + UR1_BASE)))
#define UR0_MCR (*((vu32*)(0x110 + UR1_BASE)))
#define UR0_LSR (*((vu32*)(0x114 + UR1_BASE)))
#define UR0_MSR (*((vu32*)(0x118 + UR1_BASE)))
#define UR0_SCR (*((vu32*)(0x11c + UR1_BASE)))
#define UR0_SPR (*((vu32*)(0x11c + UR1_BASE))) //to fit andes's naming
#define UR0_DLL (*((vu32*)(0x100 + UR1_BASE)))
#define UR0_DLM (*((vu32*)(0x104 + UR1_BASE)))

#define UR_LCR_5BIT (0x0 << 0)
#define UR_LCR_6BIT (0x1 << 0)
#define UR_LCR_7BIT (0x2 << 0)
#define UR_LCR_8BIT (0x3 << 0)

#define UR_LCR_STB  (0x1 << 2)
#define UR_LCR_PEN  (0x1 << 3)
#define UR_LCR_EPS  (0x1 << 4)
#define UR_LCR_SPAR (0x1 << 5)
#define UR_LCR_SBRK (0x1 << 6)
#define UR_LCR_DLAB (0x1 << 7)

// UART 1 ~ 2, ATC010
// 8-bit module
#define UR1_RBR         (*((vu32*)(0x000 + UR1_BASE)))
#define UR1_THR         (*((vu32*)(0x000 + UR1_BASE)))
#define UR1_IER         (*((vu32*)(0x004 + UR1_BASE)))
#define UR1_IIR         (*((vu32*)(0x008 + UR1_BASE)))
#define UR1_FCR         (*((vu32*)(0x008 + UR1_BASE)))
#define UR1_LCR         (*((vu32*)(0x00c + UR1_BASE)))
#define UR1_MCR         (*((vu32*)(0x010 + UR1_BASE)))
#define UR1_LSR         (*((vu32*)(0x014 + UR1_BASE)))
#define UR1_TST         (*((vu32*)(0x014 + UR1_BASE)))
#define UR1_MSR         (*((vu32*)(0x018 + UR1_BASE)))
#define UR1_SPR         (*((vu32*)(0x01c + UR1_BASE)))
#define UR1_DLL         (*((vu32*)(0x000 + UR1_BASE)))
#define UR1_DLM         (*((vu32*)(0x004 + UR1_BASE)))
#define UR1_PSR         (*((vu32*)(0x008 + UR1_BASE)))
#define UR1_MDR         (*((vu32*)(0x020 + UR1_BASE)))
#define UR1_ACR         (*((vu32*)(0x024 + UR1_BASE)))
#define UR1_TXLENL      (*((vu32*)(0x028 + UR1_BASE)))
#define UR1_TXLENH      (*((vu32*)(0x02c + UR1_BASE)))
#define UR1_MRXLENL     (*((vu32*)(0x030 + UR1_BASE)))
#define UR1_MRXLENH     (*((vu32*)(0x034 + UR1_BASE)))
#define UR1_PLR         (*((vu32*)(0x038 + UR1_BASE)))
#define UR1_FMIIR_PIO   (*((vu32*)(0x03c + UR1_BASE)))
#define UR1_FMIIR_DMA   (*((vu32*)(0x03c + UR1_BASE)))
#define UR1_FMIIER_PIO  (*((vu32*)(0x040 + UR1_BASE)))
#define UR1_FMIIER_DMA  (*((vu32*)(0x040 + UR1_BASE)))
#define UR1_STFF_STS    (*((vu32*)(0x044 + UR1_BASE)))
#define UR1_STFF_RXLENL (*((vu32*)(0x048 + UR1_BASE)))
#define UR1_STFF_RXLENH (*((vu32*)(0x04c + UR1_BASE)))
#define UR1_FMLSR       (*((vu32*)(0x050 + UR1_BASE)))
#define UR1_FMLSIER     (*((vu32*)(0x054 + UR1_BASE)))
#define UR1_RSR         (*((vu32*)(0x058 + UR1_BASE)))
#define UR1_RXFF_CNTR   (*((vu32*)(0x05c + UR1_BASE)))
#define UR1_LSTFMLENL   (*((vu32*)(0x060 + UR1_BASE)))
#define UR1_LSTFMLENH   (*((vu32*)(0x064 + UR1_BASE)))
#define UR1_FEATURE     (*((vu32*)(0x068 + UR1_BASE)))
#define UR1_REVD1       (*((vu32*)(0x06c + UR1_BASE)))
#define UR1_REVD2       (*((vu32*)(0x070 + UR1_BASE)))
#define UR1_REVD3       (*((vu32*)(0x074 + UR1_BASE)))

//LCDC
#define LCDC_BASE       (CPE_LCD_BASE)

#define LCDC_TIMING0    (*((vu32*)(0x0000 + LCDC_BASE)))
#define LCDC_TIMING1    (*((vu32*)(0x0004 + LCDC_BASE)))
#define LCDC_TIMING2    (*((vu32*)(0x0008 + LCDC_BASE)))
#define LCDC_FRAMEBASE  (*((vu32*)(0x0010 + LCDC_BASE)))
#define LCDC_INTENABLE  (*((vu32*)(0x0018 + LCDC_BASE)))
#define LCDC_CONTROL    (*((vu32*)(0x001c + LCDC_BASE)))
#define LCDC_INTCLR     (*((vu32*)(0x0020 + LCDC_BASE)))
#define LCDC_INTERRUPT  (*((vu32*)(0x0024 + LCDC_BASE)))
#define LCDC_OSDCON0    (*((vu32*)(0x0034 + LCDC_BASE)))
#define LCDC_OSDCON1    (*((vu32*)(0x0038 + LCDC_BASE)))
#define LCDC_OSDCON2    (*((vu32*)(0x003c + LCDC_BASE)))
#define LCDC_OSDCON3    (*((vu32*)(0x0040 + LCDC_BASE)))
#define LCDC_GPIOCON    (*((vu32*)(0x0044 + LCDC_BASE)))
#define LCDC_PALETTE    (*((vu32*)(0x0200 + LCDC_BASE)))
#define LCDC_OSDFNTBASE (*((vu32*)(0x8000 + LCDC_BASE)))
#define LCDC_OSDATRBASE (*((vu32*)(0xc000 + LCDC_BASE)))

//ETHC
#define ETHC_BASE (CPE_FTMAC_BASE)

#define ETHC_ISR                (*((vu32*)(0x00 + ETHC_BASE)))
#define ETHC_IMR                (*((vu32*)(0x04 + ETHC_BASE)))
#define ETHC_MAC_MADR           (*((vu32*)(0x08 + ETHC_BASE)))
#define ETHC_MAC_LADR           (*((vu32*)(0x0c + ETHC_BASE)))
#define ETHC_MAHT0              (*((vu32*)(0x10 + ETHC_BASE)))
#define ETHC_MAHT1              (*((vu32*)(0x14 + ETHC_BASE)))
#define ETHC_TXPD               (*((vu32*)(0x18 + ETHC_BASE)))
#define ETHC_RXPD               (*((vu32*)(0x1c + ETHC_BASE)))
#define ETHC_TXR_BADR           (*((vu32*)(0x20 + ETHC_BASE)))
#define ETHC_RXR_BADR           (*((vu32*)(0x24 + ETHC_BASE)))
#define ETHC_ITC                (*((vu32*)(0x28 + ETHC_BASE)))
#define ETHC_APTC               (*((vu32*)(0x2c + ETHC_BASE)))
#define ETHC_DBLAC              (*((vu32*)(0x30 + ETHC_BASE)))
#define ETHC_MACCR              (*((vu32*)(0x88 + ETHC_BASE)))
#define ETHC_MACSR              (*((vu32*)(0x8c + ETHC_BASE)))
#define ETHC_PHYCR              (*((vu32*)(0x90 + ETHC_BASE)))
#define ETHC_PHYWDATA           (*((vu32*)(0x94 + ETHC_BASE)))
#define ETHC_FCR                (*((vu32*)(0x98 + ETHC_BASE)))
#define ETHC_BPR                (*((vu32*)(0x9c + ETHC_BASE)))
#define ETHC_WOLCR              (*((vu32*)(0xa0 + ETHC_BASE)))
#define ETHC_WOLSR              (*((vu32*)(0xa4 + ETHC_BASE)))
#define ETHC_WFCRC              (*((vu32*)(0xa8 + ETHC_BASE)))
#define ETHC_WFBM1              (*((vu32*)(0xb0 + ETHC_BASE)))
#define ETHC_WFBM2              (*((vu32*)(0xb4 + ETHC_BASE)))
#define ETHC_WFBM3              (*((vu32*)(0xb8 + ETHC_BASE)))
#define ETHC_WFBM4              (*((vu32*)(0xbc + ETHC_BASE)))
#define ETHC_TS                 (*((vu32*)(0xc4 + ETHC_BASE)))
#define ETHC_DMAFIFOS           (*((vu32*)(0xc8 + ETHC_BASE)))
#define ETHC_TM                 (*((vu32*)(0xcc + ETHC_BASE)))
#define ETHC_TX_xCOL_CNTR       (*((vu32*)(0xd4 + ETHC_BASE)))
#define ETHC_RPF_AEP_CNTR       (*((vu32*)(0xd8 + ETHC_BASE)))
#define ETHC_XM_PG_CNTR         (*((vu32*)(0xdc + ETHC_BASE)))
#define ETHC_RUNT_LCC_CNTR      (*((vu32*)(0xe0 + ETHC_BASE)))
#define ETHC_CRC_ERR_FTL_CNTR   (*((vu32*)(0xe4 + ETHC_BASE)))
#define ETHC_RLC_RCC_CNTR       (*((vu32*)(0xe8 + ETHC_BASE)))
#define ETHC_BROC_CNTR          (*((vu32*)(0xec + ETHC_BASE)))
#define ETHC_MULCA_CNTR         (*((vu32*)(0xf0 + ETHC_BASE)))
#define ETHC_RP_CNTR            (*((vu32*)(0xf4 + ETHC_BASE)))
#define ETHC_XP_CNTR            (*((vu32*)(0xf8 + ETHC_BASE)))

#define ETHC_MACCR_XDMA_EN      (1 <<  0)
#define ETHC_MACCR_RDMA_EN      (1 <<  1)
#define ETHC_MACCR_SW_RST       (1 <<  2)
#define ETHC_MACCR_LOOP_EN      (1 <<  3)
#define ETHC_MACCR_CRC_DIS      (1 <<  4)
#define ETHC_MACCR_XMT_EN       (1 <<  5)
#define ETHC_MACCR_ENRX_HALFTX  (1 <<  6)
#define ETHC_MACCR_RCV_EN       (1 <<  8)
#define ETHC_MACCR_HT_MULTI_EN  (1 <<  9)
#define ETHC_MACCR_RX_RUNT      (1 << 10)
#define ETHC_MACCR_RX_FTL       (1 << 11)
#define ETHC_MACCR_RCV_ALL      (1 << 12)
#define ETHC_MACCR_CRC_APD      (1 << 14)
#define ETHC_MACCR_FULLDUP      (1 << 15)
#define ETHC_MACCR_RX_MULTIPKT  (1 << 16)
#define ETHC_MACCR_RX_BROADPKT  (1 << 17)

#define ETHC_IRQ_RPKT_FINISH    (1 <<  0)
#define ETHC_IRQ_NORXBUF        (1 <<  1)
#define ETHC_IRQ_XPKT_FINISH    (1 <<  2)
#define ETHC_IRQ_NOTXBUF        (1 <<  3)
#define ETHC_IRQ_XPKT_OK        (1 <<  4)
#define ETHC_IRQ_XPKT_LOST      (1 <<  5)
#define ETHC_IRQ_RPKT_SAV       (1 <<  6)
#define ETHC_IRQ_RPKT_LOST      (1 <<  7)
#define ETHC_IRQ_AHB_ERR        (1 <<  8)
#define ETHC_IRQ_PHYSTS_CHG     (1 <<  9)

#define TXDES0_TXPKT_LATECOL    (1 <<  0)
#define TXDES0_TXPKT_EXSCOL     (1 <<  1)
#define TXDES0_TXDMA_OWN        (1 << 31)

#define TXDES1_TXBUF_SIZE_MSK   (0x07ff << 0)
#define TXDES1_LTS              (1 << 27)
#define TXDES1_FTS              (1 << 28)
#define TXDES1_TX2FIC           (1 << 29)
#define TXDES1_TXIC             (1 << 30)
#define TXDES1_EDOTR            (1 << 31)

#define RXDES0_RFL_MSK          (0x07ff << 0)
#define RXDES0_MULTICAST        (1 << 16)
#define RXDES0_BROADCAST        (1 << 17)
#define RXDES0_RX_ERR           (1 << 18)
#define RXDES0_CRC_ERR          (1 << 19)
#define RXDES0_FTL              (1 << 20)
#define RXDES0_RUNT             (1 << 21)
#define RXDES0_RX_ODD_NB        (1 << 22)
#define RXDES0_LRS              (1 << 28)
#define RXDES0_FRS              (1 << 29)
#define RXDES0_RXDMA_OWN        (1 << 31)

#define RXDES1_RXBUF_SIZE_MSK   (0x07ff << 0)
#define RXDES1_EDORR            (1 << 31)

#endif //VM_H
